package query

import (
	"context"
	"fmt"
	"github.com/odysseus/service-registry/common"
	"github.com/odysseus/service-registry/registry"
	"github.com/redis/go-redis/v9"
	"sort"
)

type baseService struct {
	rdb     *redis.Client
	service common.ServiceType
}

func (g baseService) ModuleName() string {
	return g.service.String()
}

func (g baseService) List() ([]string, error) {
	all, err := getAll(g.rdb, g.service)
	if err != nil {
		return nil, err
	}
	sort.Sort(ServiceInfoList(all))
	var res []string
	for _, v := range all {
		res = append(res, string(v.Message()))
	}
	return res, nil
}

func (g baseService) ServiceInfo(serviceid string) (string, error) {
	info, err := getOne(g.rdb, g.service, serviceid)
	if err != nil {
		return "", err
	}
	return string(info.Message()), nil
}

func parseInfo(stype common.ServiceType, res *redis.MapStringStringCmd) ServiceInfo {
	var regInfo registry.RegistryInfo
	if err := res.Scan(&regInfo); err != nil {
		return nil
	}
	return regInfo
}

func getAll(rdb *redis.Client, serviceType common.ServiceType) (ServiceInfoList, error) {
	keys, err := rdb.Keys(context.Background(), common.GetServiceKeyPrefix(serviceType)+"*").Result()
	if err != nil {
		return nil, err
	}
	var list []ServiceInfo
	for _, key := range keys {
		res := rdb.HGetAll(context.Background(), key)
		if info := parseInfo(serviceType, res); info != nil {
			list = append(list, info)
		}
	}
	return list, nil
}

func getOne(rdb *redis.Client, serviceType common.ServiceType, endpoint string) (ServiceInfo, error) {
	k := fmt.Sprintf("%s%s", common.GetServiceKeyPrefix(serviceType), endpoint)
	res := rdb.HGetAll(context.Background(), k)
	return parseInfo(serviceType, res), nil
}
